@@ -1,16 +1,29 @@
 """This module defines specific functions for MySQL dialect."""
 
 from sqlalchemy import text
+from sqlalchemy.dialects.mysql.base import ischema_names as _mysql_ischema_names
 from sqlalchemy.ext.compiler import compiles
 from sqlalchemy.sql.sqltypes import NullType
 
 from geoalchemy2 import functions
 from geoalchemy2.admin.dialects.common import _check_spatial_type
 from geoalchemy2.admin.dialects.common import _spatial_idx_name
+from geoalchemy2.admin.dialects.common import compile_bin_literal
 from geoalchemy2.admin.dialects.common import setup_create_drop
 from geoalchemy2.types import Geography
 from geoalchemy2.types import Geometry
 
+# Register Geometry, Geography and Raster to SQLAlchemy's reflection subsystems.
+_mysql_ischema_names["geometry"] = Geometry
+_mysql_ischema_names["point"] = Geometry
+_mysql_ischema_names["linestring"] = Geometry
+_mysql_ischema_names["polygon"] = Geometry
+_mysql_ischema_names["multipoint"] = Geometry
+_mysql_ischema_names["multilinestring"] = Geometry
+_mysql_ischema_names["multipolygon"] = Geometry
+_mysql_ischema_names["geometrycollection"] = Geometry
+
+
 _POSSIBLE_TYPES = [
     "geometry",
     "point",
@@ -31,11 +44,16 @@ def reflect_geometry_column(inspector, table, column_info):
     column_name = column_info.get("name")
     schema = table.schema or inspector.default_schema_name
 
+    if inspector.dialect.name == "mariadb":
+        select_srid = "-1, "
+    else:
+        select_srid = "SRS_ID, "
+
     # Check geometry type, SRID and if the column is nullable
-    geometry_type_query = """SELECT DATA_TYPE, SRS_ID, IS_NULLABLE
+    geometry_type_query = """SELECT DATA_TYPE, {}IS_NULLABLE
         FROM INFORMATION_SCHEMA.COLUMNS
         WHERE TABLE_NAME = '{}' and COLUMN_NAME = '{}'""".format(
-        table.name, column_name
+        select_srid, table.name, column_name
     )
     if schema is not None:
         geometry_type_query += """ and table_schema = '{}'""".format(schema)
@@ -43,7 +61,7 @@ def reflect_geometry_column(inspector, table, column_info):
     is_nullable = str(nullable_str).lower() == "yes"
 
     if geometry_type not in _POSSIBLE_TYPES:
-        return
+        return  # pragma: no cover
 
     # Check if the column has spatial index
     has_index_query = """SELECT DISTINCT
@@ -67,6 +85,25 @@ def reflect_geometry_column(inspector, table, column_info):
     )
 
 
+def before_cursor_execute(
+    conn, cursor, statement, parameters, context, executemany, convert=True
+):  # noqa: D417
+    """Event handler to cast the parameters properly.
+
+    Args:
+        convert (bool): Trigger the conversion.
+    """
+    if convert:
+        if isinstance(parameters, (tuple, list)):
+            parameters = tuple(x.tobytes() if isinstance(x, memoryview) else x for x in parameters)
+        elif isinstance(parameters, dict):
+            for k in parameters:
+                if isinstance(parameters[k], memoryview):
+                    parameters[k] = parameters[k].tobytes()
+
+    return statement, parameters
+
+
 def before_create(table, bind, **kw):
     """Handle spatial indexes during the before_create event."""
     dialect, gis_cols, regular_cols = setup_create_drop(table, bind)
@@ -100,6 +137,7 @@ def after_create(table, bind, **kw):
         if (
             _check_spatial_type(col.type, (Geometry, Geography), dialect)
             and col.type.spatial_index is True
+            and col.computed is None
         ):
             # If the index does not exist, define it and create it
             if not [i for i in table.indexes if col in i.columns.values()]:
@@ -119,9 +157,7 @@ def after_drop(table, bind, **kw):
     return
 
 
-_MYSQL_FUNCTIONS = {
-    "ST_AsEWKB": "ST_AsBinary",
-}
+_MYSQL_FUNCTIONS = {"ST_AsEWKB": "ST_AsBinary", "ST_SetSRID": "ST_SRID"}
 
 
 def _compiles_mysql(cls, fn):
@@ -129,7 +165,6 @@ def _compiles_mysql(cls, fn):
         return "{}({})".format(fn, compiler.process(element.clauses, **kw))
 
     compiles(getattr(functions, cls), "mysql")(_compile_mysql)
-    compiles(getattr(functions, cls), "mariadb")(_compile_mysql)
 
 
 def register_mysql_mapping(mapping):
@@ -152,49 +187,56 @@ register_mysql_mapping(_MYSQL_FUNCTIONS)
 
 
 def _compile_GeomFromText_MySql(element, compiler, **kw):
-    element.identifier = "ST_GeomFromText"
+    identifier = "ST_GeomFromText"
     compiled = compiler.process(element.clauses, **kw)
     srid = element.type.srid
 
     if srid > 0:
-        return "{}({}, {})".format(element.identifier, compiled, srid)
+        return "{}({}, {})".format(identifier, compiled, srid)
     else:
-        return "{}({})".format(element.identifier, compiled)
+        return "{}({})".format(identifier, compiled)
 
 
 def _compile_GeomFromWKB_MySql(element, compiler, **kw):
-    element.identifier = "ST_GeomFromWKB"
-    wkb_data = list(element.clauses)[0].value
-    if isinstance(wkb_data, memoryview):
-        list(element.clauses)[0].value = wkb_data.tobytes()
-    compiled = compiler.process(element.clauses, **kw)
-    srid = element.type.srid
+    # Store the SRID
+    clauses = list(element.clauses)
+    try:
+        srid = clauses[1].value
+    except (IndexError, TypeError, ValueError):
+        srid = element.type.srid
+
+    if kw.get("literal_binds", False):
+        wkb_clause = compile_bin_literal(clauses[0])
+        prefix = "unhex("
+        suffix = ")"
+    else:
+        wkb_clause = clauses[0]
+        prefix = ""
+        suffix = ""
+
+    compiled = compiler.process(wkb_clause, **kw)
 
     if srid > 0:
-        return "{}({}, {})".format(element.identifier, compiled, srid)
+        return "{}({}{}{}, {})".format(element.identifier, prefix, compiled, suffix, srid)
     else:
-        return "{}({})".format(element.identifier, compiled)
+        return "{}({}{}{})".format(element.identifier, prefix, compiled, suffix)
 
 
 @compiles(functions.ST_GeomFromText, "mysql")  # type: ignore
-@compiles(functions.ST_GeomFromText, "mariadb")  # type: ignore
 def _MySQL_ST_GeomFromText(element, compiler, **kw):
     return _compile_GeomFromText_MySql(element, compiler, **kw)
 
 
 @compiles(functions.ST_GeomFromEWKT, "mysql")  # type: ignore
-@compiles(functions.ST_GeomFromEWKT, "mariadb")  # type: ignore
 def _MySQL_ST_GeomFromEWKT(element, compiler, **kw):
     return _compile_GeomFromText_MySql(element, compiler, **kw)
 
 
 @compiles(functions.ST_GeomFromWKB, "mysql")  # type: ignore
-@compiles(functions.ST_GeomFromWKB, "mariadb")  # type: ignore
 def _MySQL_ST_GeomFromWKB(element, compiler, **kw):
     return _compile_GeomFromWKB_MySql(element, compiler, **kw)
 
 
 @compiles(functions.ST_GeomFromEWKB, "mysql")  # type: ignore
-@compiles(functions.ST_GeomFromEWKB, "mariadb")  # type: ignore
 def _MySQL_ST_GeomFromEWKB(element, compiler, **kw):
     return _compile_GeomFromWKB_MySql(element, compiler, **kw)
