@@ -11,6 +11,7 @@ from sqlalchemy.types import TypeDecorator
 from geoalchemy2 import Geography
 from geoalchemy2 import Geometry
 from geoalchemy2 import Raster
+from geoalchemy2.elements import WKTElement
 
 
 @pytest.fixture
@@ -126,7 +127,22 @@ class TransformedGeometry(TypeDecorator):
         )
 
     def bind_expression(self, bindvalue):
-        return func.ST_Transform(self.impl.bind_expression(bindvalue), self.db_srid)
+        return func.ST_Transform(func.ST_GeomFromText(bindvalue, self.app_srid), self.db_srid)
+
+    def bind_processor(self, dialect):
+        """Specific bind_processor that automatically process spatial elements.
+
+        Here we only use WKT representations.
+        """
+
+        def process(bindvalue):
+            bindvalue = WKTElement(bindvalue)
+            bindvalue = bindvalue.as_wkt()
+            if bindvalue.srid <= 0:
+                bindvalue.srid = self.srid
+            return bindvalue.desc
+
+        return process
 
 
 @pytest.fixture
@@ -189,9 +205,16 @@ def reflection_tables_metadata(dialect_name):
             geom_no_idx = Column(
                 Geometry(geometry_type="LINESTRING", srid=4326, spatial_index=False)
             )
-            if dialect_name != "mysql":
+            if dialect_name not in ["mysql", "mariadb"]:
                 geom_z = Column(Geometry(geometry_type="LINESTRINGZ", srid=4326, dimension=3))
                 geom_m = Column(Geometry(geometry_type="LINESTRINGM", srid=4326, dimension=3))
                 geom_zm = Column(Geometry(geometry_type="LINESTRINGZM", srid=4326, dimension=4))
+            if dialect_name in ["postgresql"]:
+                geom_geog = Column(Geography(geometry_type="LINESTRING"))
+                geom_geog_no_idx = Column(
+                    Geography(geometry_type="LINESTRING", spatial_index=False)
+                )
+                rast = Column(Raster(spatial_index=True))
+                rast_no_idx = Column(Raster(spatial_index=False))
 
     return metadata
