@@ -11,11 +11,13 @@ else:
     compat.register()
     del compat
 
-from pkg_resources import parse_version
+import shapely
+from packaging.version import parse as parse_version
 from shapely.geometry import LineString
 from shapely.geometry import Point
 from sqlalchemy import CheckConstraint
 from sqlalchemy import Column
+from sqlalchemy import Computed
 from sqlalchemy import Integer
 from sqlalchemy import MetaData
 from sqlalchemy import String
@@ -54,6 +56,7 @@ from . import skip_postgis1
 from . import test_only_with_dialects
 
 SQLA_LT_2 = parse_version(SA_VERSION) <= parse_version("1.999")
+SHAPELY_LT_21 = parse_version(shapely.__version__) < parse_version("2.1")
 
 
 class TestAdmin:
@@ -229,6 +232,105 @@ class TestAdmin:
 
         assert marks == ["before_create", "after_create", "before_drop", "after_drop"]
 
+    def test_computed_column_core(self, conn):
+        meta = MetaData()
+
+        # Define the table
+        t = Table(
+            "computed_column",
+            meta,
+            Column("id", Integer, primary_key=True),
+            Column("x", Integer),
+            Column("y", Integer),
+            Column(
+                "computed_geom",
+                Geometry(geometry_type="POINT", srid=4326),
+                Computed("ST_POINT(x, y)", persisted=True),
+            ),
+        )
+
+        # Create the table
+        meta.create_all(bind=conn)
+
+        conn.execute(
+            t.insert(),
+            [
+                {"x": "1", "y": "2"},
+                {"x": "2", "y": "3"},
+            ],
+        )
+
+        res = conn.execute(
+            select(
+                [
+                    t.c.id,
+                    t.c.x,
+                    t.c.y,
+                    t.c.computed_geom.ST_AsText(),
+                ]
+            ),
+        ).fetchall()
+
+        # Check inserted data
+        for i in res:
+            x = i[1]
+            y = i[2]
+            p_x, p_y = [int(j) for j in re.findall(r"\d+", i[3])]
+            assert x == p_x
+            assert y == p_y
+
+        # Drop the table
+        meta.drop_all(bind=conn)
+
+    def test_computed_column_orm(self, conn, base, metadata):
+
+        # Define the table
+        class ComputedGeomTable(base):
+            __tablename__ = "computed_column"
+            id = Column(Integer, primary_key=True)
+            x = Column(Integer)
+            y = Column(Integer)
+            computed_geom = Column(
+                Geometry(geometry_type="POINT", srid=4326),
+                Computed("ST_POINT(x, y)", persisted=True),
+            )
+
+            def __init__(self, computed_geom):
+                self.computed_geom = computed_geom
+
+        # Create the table
+        metadata.create_all(bind=conn)
+
+        conn.execute(
+            ComputedGeomTable.__table__.insert(),
+            [
+                {"x": "1", "y": "2"},
+                {"x": "2", "y": "3"},
+            ],
+        )
+
+        res = conn.execute(
+            select(
+                [
+                    ComputedGeomTable.__table__.c.id,
+                    ComputedGeomTable.__table__.c.x,
+                    ComputedGeomTable.__table__.c.y,
+                    ComputedGeomTable.__table__.c.computed_geom.ST_AsText(),
+                ]
+            ),
+        ).fetchall()
+
+        # Check inserted data
+        for i in res:
+            x = i[1]
+            y = i[2]
+            p_x, p_y = [int(j) for j in re.findall(r"\d+", i[3])]
+            assert x == p_x
+            assert y == p_y
+
+        # Drop the table
+        metadata.drop_all(bind=conn)
+
 
 class TestInsertionCore:
     def test_insert(self, conn, Lake, setup_tables):
@@ -237,26 +339,30 @@ class TestInsertionCore:
         conn.execute(
             Lake.__table__.insert(),
             [
-                {"geom": "SRID=4326;LINESTRING(0 0,1 1)"},
-                {"geom": WKTElement("LINESTRING(0 0,2 2)", srid=4326)},
-                {"geom": WKTElement("SRID=4326;LINESTRING(0 0,2 2)", extended=True)},
-                {"geom": from_shape(LineString([[0, 0], [3, 3]]), srid=4326)},
+                {"id": 1, "geom": "SRID=4326;LINESTRING(0 0,1 1)"},
+                {"id": 2, "geom": WKTElement("LINESTRING(0 0,2 2)", srid=4326)},
+                {"id": 3, "geom": WKTElement("SRID=4326;LINESTRING(0 0,2 2)", extended=True)},
+                {"id": 4, "geom": from_shape(LineString([[0, 0], [3, 3]]), srid=4326)},
             ],
         )
 
-        results = conn.execute(Lake.__table__.select())
+        results = conn.execute(Lake.__table__.select().order_by("id"))
         rows = results.fetchall()
 
         row = rows[0]
         assert isinstance(row[1], WKBElement)
-        wkt = conn.execute(row[1].ST_AsText()).scalar()
+        wkt = conn.execute(from_shape(LineString([[0, 0], [3, 3]]), srid=4326).ST_AsText()).scalar()
+        q1 = row[1].ST_AsText()
+        wkt = conn.execute(q1).scalar()
         assert format_wkt(wkt) == "LINESTRING(0 0,1 1)"
         srid = conn.execute(row[1].ST_SRID()).scalar()
         assert srid == 4326
 
         row = rows[1]
         assert isinstance(row[1], WKBElement)
-        wkt = conn.execute(row[1].ST_AsText()).scalar()
+        q2 = row[1].ST_AsText()
+        wkt = conn.execute(q2).scalar()
+
         assert format_wkt(wkt) == "LINESTRING(0 0,2 2)"
         srid = conn.execute(row[1].ST_SRID()).scalar()
         assert srid == 4326
@@ -355,7 +461,7 @@ class TestInsertionCore:
         else:
             has_m = False
 
-        if ndims > 2 and dialect_name == "mysql":
+        if ndims > 2 and dialect_name in ["mysql", "mariadb"]:
             # Explicitly skip MySQL dialect to show that it can only work with 2D geometries
             pytest.xfail(reason="MySQL only supports 2D geometry types")
 
@@ -372,12 +478,6 @@ class TestInsertionCore:
 
         inserted_wkt = f"{geom_type}{wkt}"
 
-        # Use the DB to generate the corresponding raw WKB
-        raw_wkb = conn.execute(
-            text("SELECT ST_AsBinary(ST_GeomFromText('{}', 4326))".format(inserted_wkt))
-        ).scalar()
-
-        wkb_elem = WKBElement(raw_wkb, srid=4326)
         inserted_elements = [
             {"geom": inserted_wkt},
             {"geom": f"SRID=4326;{inserted_wkt}"},
@@ -385,10 +485,20 @@ class TestInsertionCore:
             {"geom": WKTElement(f"SRID=4326;{inserted_wkt}")},
         ]
         if dialect_name not in ["postgresql", "sqlite"] or not has_m:
+            # Use the DB to generate the corresponding raw WKB
+            raw_wkb = conn.execute(
+                text("SELECT ST_AsBinary(ST_GeomFromText('{}', 4326))".format(inserted_wkt))
+            ).scalar()
+
+            wkb_elem = WKBElement(raw_wkb, srid=4326)
+
             # Currently Shapely does not support geometry types with M dimension
             inserted_elements.append({"geom": wkb_elem})
             inserted_elements.append({"geom": wkb_elem.as_ewkb()})
 
+        print(inserted_elements)
+        # raise ValueError()
+
         # Insert the elements
         conn.execute(
             GeomTypeTable.__table__.insert(),
@@ -414,13 +524,76 @@ class TestInsertionCore:
                 # Some dialects return MULTIPOINT geometries with nested parenthesis and others
                 # do not so we remove them before checking the results
                 checked_wkt = re.sub(r"\(([0-9\.]+)\)", "\\1", checked_wkt)
-            if row_id >= 5 and dialect_name in ["geopackage"] and has_m:
+            if row_id >= 5 and dialect_name in ["geopackage"] and has_m and SHAPELY_LT_21:
                 # Currently Shapely does not support geometry types with M dimension
                 assert checked_wkt != expected_wkt
             else:
                 assert checked_wkt == expected_wkt
             assert srid == 4326
 
+    @pytest.mark.parametrize(
+        "geom_type",
+        [
+            pytest.param("POINT", id="Point"),
+            pytest.param("POINTZ", id="Point Z"),
+            pytest.param("POINTM", id="Point M"),
+            pytest.param("POINTZM", id="Point ZM"),
+            pytest.param("LINESTRING", id="LineString"),
+            pytest.param("LINESTRINGZ", id="LineString Z"),
+            pytest.param("LINESTRINGM", id="LineString M"),
+            pytest.param("LINESTRINGZM", id="LineString ZM"),
+            pytest.param("POLYGON", id="Polygon"),
+            pytest.param("POLYGONZ", id="Polygon Z"),
+            pytest.param("POLYGONM", id="Polygon M"),
+            pytest.param("POLYGONZM", id="Polygon ZM"),
+            pytest.param("MULTIPOINT", id="Multi Point"),
+            pytest.param("MULTIPOINTZ", id="Multi Point Z"),
+            pytest.param("MULTIPOINTM", id="Multi Point M"),
+            pytest.param("MULTIPOINTZM", id="Multi Point ZM"),
+            pytest.param("MULTILINESTRING", id="Multi LineString"),
+            pytest.param("MULTILINESTRINGZ", id="Multi LineString Z"),
+            pytest.param("MULTILINESTRINGM", id="Multi LineString M"),
+            pytest.param("MULTILINESTRINGZM", id="Multi LineString ZM"),
+            pytest.param("MULTIPOLYGON", id="Multi Polygon"),
+            pytest.param("MULTIPOLYGONZ", id="Multi Polygon Z"),
+            pytest.param("MULTIPOLYGONM", id="Multi Polygon M"),
+            pytest.param("MULTIPOLYGONZM", id="Multi Polygon ZM"),
+        ],
+    )
+    @pytest.mark.parametrize(
+        "dimension",
+        [
+            pytest.param(
+                None,
+                id="Dimension is None",
+            ),
+            pytest.param(
+                -1,
+                id="Negative dimension",
+            ),
+            pytest.param(
+                1,
+                id="Wrong dimension",
+            ),
+        ],
+    )
+    def test_check_ctor_args(self, dialect_name, base, conn, metadata, geom_type, dimension):
+        ndims = 2
+        if "Z" in geom_type[-2:]:
+            ndims += 1
+        if geom_type.endswith("M"):
+            ndims += 1
+
+        if ndims > 2 and dialect_name in ["mysql", "mariadb"]:
+            # Explicitly skip MySQL dialect to show that it can only work with 2D geometries
+            pytest.xfail(reason="MySQL only supports 2D geometry types")
+
+        if dimension is not None:
+            with pytest.raises(ValueError):
+                Geometry(srid=4326, geometry_type=geom_type, dimension=dimension)
+        else:
+            Geometry(srid=4326, geometry_type=geom_type, dimension=dimension)
+
     @test_only_with_dialects("postgresql", "sqlite")
     def test_insert_geom_poi(self, conn, Poi, setup_tables):
         conn.execute(
@@ -466,10 +639,8 @@ class TestInsertionCore:
             assert format_wkt(wkt) == "POINT(-1 1)"
             srid = conn.execute(row[1].ST_SRID()).scalar()
             assert srid == 4326
-            if dialect_name == "mysql":
-                assert row[1] == from_shape(Point(-1, 1), srid=4326)
-            else:
-                assert row[1] == from_shape(Point(-1, 1), srid=4326, extended=True)
+            extended = dialect_name not in ["mysql", "mariadb"]
+            assert row[1] == from_shape(Point(-1, 1), srid=4326, extended=extended)
 
 
 class TestSelectBindParam:
@@ -516,7 +687,7 @@ class TestSelectBindParam:
         rows = results.fetchall()
         geom = rows[0][1]
         assert isinstance(geom, WKBElement)
-        if dialect_name == "mysql":
+        if dialect_name in ["mysql", "mariadb"]:
             assert geom.extended is False
         else:
             assert geom.extended is True
@@ -564,7 +735,7 @@ class TestInsertionORM:
         session.flush()
         session.expire(lake)
         assert isinstance(lake.geom, WKBElement)
-        if dialect_name == "mysql":
+        if dialect_name in ["mysql", "mariadb"]:
             # Not extended case
             assert str(lake.geom) == (
                 "0102000000020000000000000000000000000000000000000000000"
@@ -587,7 +758,7 @@ class TestInsertionORM:
         session.flush()
         session.expire(lake)
         assert isinstance(lake.geom, WKBElement)
-        if dialect_name == "mysql":
+        if dialect_name in ["mysql", "mariadb"]:
             # Not extended case
             assert str(lake.geom) == (
                 "0102000000020000000000000000000000000000000000000000000"
@@ -604,19 +775,18 @@ class TestInsertionORM:
         assert srid == 4326
 
     @test_only_with_dialects("postgresql", "mysql", "sqlite-spatialite3", "sqlite-spatialite4")
-    def test_transform(self, session, LocalPoint, setup_tables):
-        if session.bind.dialect.name == "mysql":
-            # Explicitly skip MySQL dialect to show that there is an issue
-            pytest.skip(
-                reason=(
-                    "The SRID is not properly retrieved so an exception is raised. TODO: This "
-                    "should be fixed later"
-                )
-            )
+    def test_transform(self, session, LocalPoint, setup_tables, dialect_name):
         # Create new point instance
         p = LocalPoint()
-        p.geom = "SRID=4326;POINT(5 45)"  # Insert geometry with wrong SRID
-        p.managed_geom = "SRID=4326;POINT(5 45)"  # Insert geometry with wrong SRID
+        if dialect_name in ["mysql", "mariadb"]:
+            expected_x = 45
+            expected_y = 5
+        else:
+            expected_x = 5
+            expected_y = 45
+        ewkt = f"SRID=4326;POINT({expected_x} {expected_y})"
+        p.geom = ewkt  # Insert geometry with wrong SRID
+        p.managed_geom = ewkt  # Insert geometry with wrong SRID
 
         # Insert point
         session.add(p)
@@ -629,11 +799,11 @@ class TestInsertionORM:
         assert pt.geom.srid == 4326
         assert pt.managed_geom.srid == 4326
         pt_wkb = to_shape(pt.geom)
-        assert round(pt_wkb.x, 5) == 5
-        assert round(pt_wkb.y, 5) == 45
+        assert round(pt_wkb.x, 5) == expected_x
+        assert round(pt_wkb.y, 5) == expected_y
         pt_wkb = to_shape(pt.managed_geom)
-        assert round(pt_wkb.x, 5) == 5
-        assert round(pt_wkb.y, 5) == 45
+        assert round(pt_wkb.x, 5) == expected_x
+        assert round(pt_wkb.y, 5) == expected_y
 
         # Check that the data is correct in DB using raw query
         q = text(
@@ -647,7 +817,10 @@ class TestInsertionORM:
         for i in [res_q.geom, res_q.managed_geom]:
             x, y = re.match(r"POINT\((\d+\.\d*) (\d+\.\d*)\)", i).groups()
             assert round(float(x), 3) == 857581.899
-            assert round(float(y), 3) == 6435414.748
+            if dialect_name in ["mysql", "mariadb"]:
+                assert round(float(y), 3) == 6434180.796
+            else:
+                assert round(float(y), 3) == 6435414.748
 
 
 class TestUpdateORM:
@@ -666,7 +839,7 @@ class TestUpdateORM:
         srid = session.execute(lake.geom.ST_SRID()).scalar()
         assert srid == 4326
 
-        if dialect_name != "mysql":
+        if dialect_name not in ["mysql", "mariadb"]:
             # Set geometry to None
             lake.geom = None
 
@@ -708,7 +881,7 @@ class TestUpdateORM:
         srid = session.execute(lake.geom.ST_SRID()).scalar()
         assert srid == 4326
 
-        if dialect_name != "mysql":
+        if dialect_name not in ["mysql", "mariadb"]:
             # Set geometry to None
             lake.geom = None
 
@@ -759,7 +932,7 @@ class TestUpdateORM:
             session.flush()
             session.refresh(lake)
             assert lake.geom is None
-        elif dialect_name == "mysql":
+        elif dialect_name in ["mysql", "mariadb"]:
             with pytest.raises(OperationalError):
                 session.flush()
         else:
@@ -925,7 +1098,7 @@ class TestShapely:
     def test_to_shape(self, session, Lake, setup_tables, dialect_name):
         if dialect_name in ["sqlite", "geopackage"]:
             data_type = str
-        elif dialect_name == "mysql":
+        elif dialect_name in ["mysql", "mariadb"]:
             data_type = bytes
         else:
             data_type = memoryview
@@ -1059,6 +1232,31 @@ class TestReflection:
                 assert type_.srid == 4326
                 assert type_.dimension == 4
 
+            if dialect_name == "postgresql":
+                type_ = t.c.geom_geog.type
+                assert isinstance(type_, Geography)
+                assert type_.geometry_type == "LINESTRING"
+                assert type_.srid == 4326
+                assert type_.dimension == 2
+
+                type_ = t.c.geom_geog_no_idx.type
+                assert isinstance(type_, Geography)
+                assert type_.geometry_type == "LINESTRING"
+                assert type_.srid == 4326
+                assert type_.dimension == 2
+
+                type_ = t.c.rast.type
+                assert isinstance(type_, Raster)
+                assert type_.geometry_type is None
+                assert type_.srid == -1
+                assert type_.dimension is None
+
+                type_ = t.c.rast_no_idx.type
+                assert isinstance(type_, Raster)
+                assert type_.geometry_type is None
+                assert type_.srid == -1
+                assert type_.dimension is None
+
         # Drop the table
         t.drop(bind=conn)
 
@@ -1110,6 +1308,10 @@ class TestReflection:
                         "idx_lake_geom",
                         "CREATE INDEX idx_lake_geom ON gis.lake USING gist (geom)",
                     ),
+                    (
+                        "idx_lake_geom_geog",
+                        "CREATE INDEX idx_lake_geom_geog ON gis.lake USING gist (geom_geog)",
+                    ),
                     (
                         "idx_lake_geom_m",
                         "CREATE INDEX idx_lake_geom_m ON gis.lake USING gist (geom_m)",
@@ -1122,6 +1324,10 @@ class TestReflection:
                         "idx_lake_geom_zm",
                         "CREATE INDEX idx_lake_geom_zm ON gis.lake USING gist (geom_zm)",
                     ),
+                    (
+                        "idx_lake_rast",
+                        "CREATE INDEX idx_lake_rast ON gis.lake USING gist (st_convexhull(rast))",
+                    ),
                     (
                         "lake_pkey",
                         "CREATE UNIQUE INDEX lake_pkey ON gis.lake USING btree (id)",
@@ -1195,3 +1401,89 @@ class TestToMetadata(ComparesTables):
 
         # Check that the spatial index was not duplicated
         assert len(new_Lake.indexes) == 1
+
+
+class TestAsBinaryWKT:
+    def test_create_insert(self, conn, dialect_name):
+        class GeometryWkt(Geometry):
+            """Geometry type that uses WKT strings."""
+
+            from_text = "ST_GeomFromEWKT"
+            as_binary = "ST_AsText"
+            ElementType = WKTElement
+
+        dialects_with_srid = ["geopackage", "mysql", "mariadb"]
+
+        # Define the table
+        cols = [
+            Column("id", Integer, primary_key=True),
+        ]
+        cols.append(Column("geom_with_srid", GeometryWkt(geometry_type="LINESTRING", srid=4326)))
+        if dialect_name not in dialects_with_srid:
+            cols.append(Column("geom", GeometryWkt(geometry_type="LINESTRING")))
+        t = Table("use_wkt", MetaData(), *cols)
+
+        # Create the table
+        t.drop(bind=conn, checkfirst=True)
+        t.create(bind=conn)
+
+        # Test element insertion
+        inserted_values = [
+            {"geom_with_srid": v}
+            for v in [
+                "SRID=4326;LINESTRING(0 0,1 1)",
+                WKTElement("LINESTRING(0 0,2 2)", srid=4326),
+                WKTElement("SRID=4326;LINESTRING(0 0,3 3)", extended=True),
+                from_shape(LineString([[0, 0], [4, 4]]), srid=4326),
+            ]
+        ]
+        if dialect_name not in dialects_with_srid:
+            for i, v in zip(
+                inserted_values,
+                [
+                    "LINESTRING(0 0,1 1)",
+                    WKTElement("LINESTRING(0 0,2 2)"),
+                    WKTElement("SRID=-1;LINESTRING(0 0,3 3)", extended=True),
+                    from_shape(LineString([[0, 0], [4, 4]])),
+                ],
+            ):
+                i["geom"] = v
+
+        conn.execute(t.insert(), inserted_values)
+
+        results = conn.execute(t.select())
+        rows = results.fetchall()
+
+        for row_num, row in enumerate(rows):
+            for num, element in enumerate(row[1:]):
+                assert isinstance(element, WKTElement)
+                wkt = conn.execute(element.ST_AsText()).scalar()
+                assert format_wkt(wkt) == f"LINESTRING(0 0,{row_num + 1} {row_num + 1})"
+                srid = conn.execute(element.ST_SRID()).scalar()
+                if num == 1:
+                    assert srid == 0 if dialect_name != "sqlite" else -1
+                else:
+                    assert srid == 4326
+
+        # Drop the table
+        t.drop(bind=conn)
+
+
+class TestCompileQuery:
+    def test_compile_query(self, conn):
+        wkb = b"\x01\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00\xf0?\x00\x00\x00\x00\x00\x00\x00@"
+        elem = WKBElement(wkb)
+        query = select([func.ST_AsText(elem)])
+        compiled_with_literal = str(query.compile(conn, compile_kwargs={"literal_binds": True}))
+        res_text = conn.execute(text(compiled_with_literal)).scalar()
+        assert res_text == "POINT(1 2)"
+
+        compiled_without_literal = str(query.compile(conn, compile_kwargs={"literal_binds": False}))
+
+        res_query = conn.execute(query).scalar()
+        assert res_query == "POINT(1 2)"
+
+        assert compiled_with_literal.startswith("SELECT ST_AsText(")
+        assert "0101000000000000000000f03f0000000000000040" in compiled_with_literal
+        assert compiled_without_literal.startswith("SELECT ST_AsText(")
+        assert "0101000000000000000000f03f0000000000000040" not in compiled_without_literal
