@@ -1,16 +1,32 @@
+import pytest
 import shapely.wkb
 from shapely.geometry import Point
 
+import geoalchemy2.shape
 from geoalchemy2.elements import WKBElement
 from geoalchemy2.elements import WKTElement
 from geoalchemy2.shape import from_shape
 from geoalchemy2.shape import to_shape
 
 
+def test_check_shapely(monkeypatch):
+
+    @geoalchemy2.shape.check_shapely()
+    def f():
+        return "ok"
+
+    assert f() == "ok"
+
+    with monkeypatch.context() as m:
+        m.setattr(geoalchemy2.shape, "HAS_SHAPELY", False)
+        with pytest.raises(ImportError):
+            f()
+
+
 def test_to_shape_WKBElement():
     # POINT(1 2)
     e = WKBElement(
-        b"\x01\x01\x00\x00\x00\x00\x00\x00\x00\x00" b"\x00\xf0?\x00\x00\x00\x00\x00\x00\x00@"
+        b"\x01\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00\xf0?\x00\x00\x00\x00\x00\x00\x00@"
     )
     s = to_shape(e)
     assert isinstance(s, Point)
@@ -58,6 +74,11 @@ def test_to_shape_WKTElement():
     assert s.y == 2
 
 
+def test_to_shape_wrong_type():
+    with pytest.raises(TypeError, match="Only WKBElement and WKTElement objects are supported"):
+        to_shape(0)
+
+
 def test_from_shape():
     # Standard case: POINT(1 2)
     expected = WKBElement(str("0101000000000000000000f03f0000000000000040"))
